SystemOrganization addCategory: #'Seaside-RenderLoop'!

Error subclass: #WAResponseExpectedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WAResponseExpectedError methodsFor: 'description' stamp: 'jf 10/18/2008 11:07'!
possibleCauses
	^ #(
		'You failed to return a response during processing of an XML HTTP (Ajax) request.'
		'You attempted to use #call: in an XML HTTP (Ajax) request.'
	)! !

!WASession methodsFor: '*Seaside-RenderLoop' stamp: 'jf 10/17/2008 20:44'!
inNewRenderLoopCall: aComponent
	^ WAContinuation currentDo: [ :cc | 
		aComponent onAnswer: cc.
		self inNewRenderLoopShow: aComponent ] ! !

!WASession methodsFor: '*Seaside-RenderLoop' stamp: 'jf 10/17/2008 21:05'!
inNewRenderLoopShow: aComponent
	((self preferenceAt: #renderPhaseContinuationClass) root: aComponent) value: nil! !

WASystemConfiguration subclass: #WARenderLoopConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderLoopConfiguration commentStamp: 'jf 7/22/2008 12:32' prior: 0!
WARenderLoopConfiguration defines attributes required by the Render Loop classes (see the Seaside-Core-RenderLoop category). Any application using these classes (e.g. WARenderLoopMain) should include this configuration (this is done by default for new applications).

See #addAttributes: for details on provided attributes.!

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 10/17/2008 20:59'!
continuationClasses
	^ WARenderLoopContinuation allSubclasses asSortedCollection: [ :a :b | a name <= b name ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 10/17/2008 21:11'!
describeOn: config
	(config list: #actionPhaseContinuationClass)
		group: #renderLoop;
		comment: 'The continuation used for the Render Loop''s action phase.';
		options: [self continuationClasses];
		default: WAActionPhaseContinuation.
	(config list: #renderPhaseContinuationClass)
		group: #renderLoop;
		comment: 'The continuation used for the Render Loop''s render phase.';
		options: [self continuationClasses];
		default: WARenderPhaseContinuation.
	(config list: #rootClass)
		comment: 'The root component of this seaside application.';
		options: [self rootClasses].
	(config collection: #rootDecorationClasses)
		comment: 'The decoration clases that should wrap the root component.';
		options: [self rootDecorationClasses].
	config
		at: #mainClass put: WARenderLoopMain! !

!WARenderLoopConfiguration methodsFor: 'ancestry' stamp: 'jf 7/24/2008 19:10'!
parents
	"We override several attributes of the session configuration."
	^ Array with: WASessionConfiguration instance! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'jf 7/29/2008 14:30'!
rootClasses
	^ (WAComponent allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

!WARenderLoopConfiguration methodsFor: 'description' stamp: 'lr 9/14/2008 12:58'!
rootDecorationClasses
	^ (WADecoration allSubclasses select: [ :each | each canBeRoot ])
		asSortedCollection: [ :a :b | a name <= b name ]! !

WASessionContinuation subclass: #WARenderLoopContinuation
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

WARenderLoopContinuation subclass: #WAActionPhaseContinuation
	instanceVariableNames: 'renderingContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WAActionPhaseContinuation methodsFor: 'processing' stamp: 'jf 10/18/2008 11:03'!
handleRequest: aRequest 
	| renderContinuation |
	aRequest isXmlHttpRequest ifTrue:
		[ self requestContext onRespond: [ :r | states snapshot ] ].
	self withNotificationHandler:
		[ self renderingContext callbacks processWithRequest: aRequest ].
	
	aRequest isXmlHttpRequest ifTrue:
		[ ^ WAResponseExpectedError raiseSignal: 'XML HTTP requests must return a response during callback evaluation.' ].
	
	renderContinuation := self newRenderPhaseContinuation.	
	
	(self shouldRedirect: aRequest) 
		ifTrue: [ self respond: [ :response |
					renderContinuation setStates: self newStates.
					response redirectTo: renderContinuation url ] ]
		ifFalse: [ renderContinuation value: aRequest ]! !

!WAActionPhaseContinuation methodsFor: 'accessing' stamp: 'jf 9/28/2008 23:37'!
renderingContext
	^ renderingContext! !

!WAActionPhaseContinuation methodsFor: 'accessing' stamp: 'jf 9/28/2008 23:37'!
renderingContext: anObject 
	renderingContext := anObject! !

!WAActionPhaseContinuation methodsFor: 'private' stamp: 'jf 10/16/2008 16:13'!
shouldRedirect: aRequest
	^ (aRequest isGet and: [ aRequest fields includesKey: '_n' ]) not! !

!WARenderLoopContinuation class methodsFor: 'instance-creation' stamp: 'jf 9/28/2008 23:02'!
root: aComponent
	^ self new initializeWithRoot: aComponent! !

!WARenderLoopContinuation methodsFor: 'initialization' stamp: 'jf 9/28/2008 23:02'!
initializeWithRoot: aComponent
	root := aComponent! !

!WARenderLoopContinuation methodsFor: 'creational' stamp: 'jf 10/17/2008 21:07'!
newActionPhaseContinuation
	"Answer a new action phase continuation."

	^ (self session preferenceAt: #actionPhaseContinuationClass) root: self root! !

!WARenderLoopContinuation methodsFor: 'creational' stamp: 'jf 10/17/2008 21:07'!
newRenderPhaseContinuation
	"Answer a new render phase continuation."

	^ (self session preferenceAt: #renderPhaseContinuationClass) root: self root! !

!WARenderLoopContinuation methodsFor: 'accessing' stamp: 'jf 9/28/2008 23:01'!
root
	"Answer the root component instance. Usually this component is instantiated with the first reuqest."

	^ root! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'jf 9/28/2008 23:07'!
updateRoot: aHtmlRoot
	super updateRoot: aHtmlRoot.
	self root visiblePresentersDo: [ :each | each updateRoot: aHtmlRoot ]! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'jf 9/28/2008 23:07'!
updateStates: aSnapshot
	super updateStates: aSnapshot.
	self root visiblePresentersDo: [ :each |
		(each isDecoration and: [ each isDelegation ])
			ifTrue: [ each component updateStates: aSnapshot ].
		each updateStates: aSnapshot ]! !

!WARenderLoopContinuation methodsFor: 'updating' stamp: 'jf 9/28/2008 23:06'!
updateUrl: anUrl
	super updateUrl: anUrl.
	self root visiblePresentersDo: [ :each | each updateUrl: anUrl ]! !

!WARenderLoopContinuation methodsFor: 'private' stamp: 'jf 9/28/2008 23:20'!
withNotificationHandler: aBlock
	^ aBlock on: WARenderNotification do: [ :n | ]! !

WARenderLoopContinuation subclass: #WARenderPhaseContinuation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'jf 9/28/2008 23:24'!
handleRequest: aRequest 
	self withNotificationHandler: [ self render ]! !

!WARenderPhaseContinuation methodsFor: 'private' stamp: 'jf 10/17/2008 21:06'!
newContext
	| renderingContext actionContinuation |
	renderingContext := WARenderingContext new.
	actionContinuation := self newActionPhaseContinuation.
	actionContinuation
		renderingContext: renderingContext;
		setStates: self newStates.
	renderingContext actionUrl: actionContinuation url.
	
	^ renderingContext! !

!WARenderPhaseContinuation methodsFor: 'private' stamp: 'jf 9/28/2008 23:32'!
newHtmlRootWithContext: aRenderContext
	| htmlRoot |
	htmlRoot := WAHtmlRoot context: aRenderContext.
	htmlRoot bodyAttributes at: 'onload' append: 'onLoad()' separator: ';'.
	self updateRoot: htmlRoot.
	^ htmlRoot! !

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'jf 9/29/2008 01:19'!
processRendering: aResponse 
	| document htmlRoot renderingContext |
	renderingContext := self newContext.
	document := self application newDocumentOn: aResponse stream.
	renderingContext document: document.
	htmlRoot := self newHtmlRootWithContext: renderingContext.
	document open: htmlRoot.
	self root decorationChainDo: [ :each | each renderWithContext: renderingContext ].
	self writeOnLoadOn: document.
	document close: htmlRoot.
	renderingContext release! !

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'jf 9/28/2008 23:28'!
render
	self session clearJumpTo.
	self respond: [ :response | self processRendering: response ]! !

!WARenderPhaseContinuation methodsFor: 'processing' stamp: 'jf 9/28/2008 23:29'!
writeOnLoadOn: aDocument 
	| attributes |
	attributes := WAHtmlAttributes new.
	attributes 
		at: 'type'
		put: 'text/javascript'.
	aDocument 
		openTag: 'script'
		attributes: attributes
		closed: false.
	aDocument nextPutAll: '/*<!![CDATA[*/function onLoad(){'.
	self requestContext loadScripts 
		do: [ :each | aDocument nextPutAll: each seasideString ]
		separatedBy: [ aDocument nextPutAll: ';' ].
	aDocument nextPutAll: '}/*]]>*/'.
	aDocument closeTag: 'script'! !

WAMain subclass: #WARenderLoopMain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-RenderLoop'!

!WARenderLoopMain commentStamp: 'rew 10/19/2007 10:50' prior: 0!
When a new session on a WAApplication is started WARenderLoopMain initializes the application, that is it:
 
	creates the top level component of the application, 
	informs each component(WAPresenter) of the application that the session started (via WAPresenter>>initialRequest:)
	starts a WARenderLoop to handle the request
!

!WARenderLoopMain methodsFor: 'creational' stamp: 'lr 9/14/2008 13:14'!
createRoot
	^ self rootDecorationClasses
		inject: self rootClass new
		into: [ :component :decorationClass |
			component 
				addDecoration: decorationClass new;
				yourself ]! !

!WARenderLoopMain methodsFor: 'accessing' stamp: 'lr 9/14/2008 12:56'!
rootClass
	^ self application preferenceAt: #rootClass! !

!WARenderLoopMain methodsFor: 'accessing' stamp: 'lr 9/14/2008 13:03'!
rootDecorationClasses
	^ self application preferenceAt: #rootDecorationClasses! !

!WARenderLoopMain methodsFor: 'processing' stamp: 'jf 10/17/2008 21:09'!
start: aRequest
	| root |
	root := self createRoot.
	root visiblePresentersDo: [ :each | each initialRequest: aRequest ].
	((self session preferenceAt: #renderPhaseContinuationClass) root: root) value: aRequest! !
